CON

  FOUT          =  3
  ATTEN         =  4
  nSW           =  5
  nLED          =  6 
  VFB           =  9
  VINP          = 10
  AUDIO         = 14
  LPF           = 18
  SCL           = 28
  SDA           = 29

  ACK           = true
  NAK           = false

VAR

  long  base, tone, _gamma, _return_loss, _vswr, _calibration[300]
  long  stack[50]

OBJ

  f     : "Float32Full"

PUB  start | freq, i

' Start the antenna analyzer object.

  f.start
  cognew(squealer, @stack)
  ctrb := %01001 << 26 | VFB << 9 | VINP 
  frqb := 1
  outa[nLED]~~
  dira := 1 << FOUT | 1 << VFB | 1 << nLED | 1 << ATTEN | 1 << SCL
  waitcnt(cnt + clkfreq / 10)
  base := adc
  _i2c_stop

PUB calibrate | i, frq, resp, addr

' Do a 100 kHz to 30 Mhz calibration.

  repeat i from 1 to 300
    _calibration[i - 1] := f.ffloat(response(i * 100_000))

PUB save_calibration(number) | i

'Save the calibration data in file number (0 - 19) in extended EEPROM.

  if (number > 19)
    return
  repeat i from 0 to 299
    ee_wrlong($8000 + 1200 * number + (i << 2), _calibration[i])
  outa[nLED]~
  waitcnt(cnt + clkfreq / 5)
  outa[nLED]~~

PUB load_calibration(number) | i

'Load the calibration data from file number (0 - 19) in extended EEPROM.

  if (number > 19)
    return
  repeat i from 0 to 299
    _calibration[i] := ee_rdlong($8000 + 1200 * number + (i << 2))

PUB squeal(frq) | resp, active

'Output an audio tone with frequency proportional to the VSWR at frq.

  active~
  repeat
    case active
      false:
        repeat while ina[nSW]                                                                                                       
          waitcnt(cnt + clkfreq >> 4)
        repeat                                                                                                                      
        until ina[nSW]
        active~~
      true:
        repeat while ina[nSW]
          compute(response(frq), frq)
          tone := f.ftrunc(f.fmul(f.fmin(_vswr, 10.0), 1000.0)) - 1000
        set_freq(0)
        tone~
        repeat
          waitcnt(cnt + clkfreq >> 4)
        until ina[nSW]
        active~

PUB gamma(frq)

' Return the value of gamma for the last sample (frq == 0),
' or take another sample at frq and return the new value.

  if (frq)
    compute(true, frq)
  return _gamma

PUB vswr(frq)

' Return the value of vswr for the last sample (frq == 0),
' or take another sample at frq and return the new value.

  if (frq)
    compute(true, frq)
  return _vswr

PUB return_loss(frq)

' Return the value of return loss for the last sample (frq == 0),
' or take another sample at frq and return the new value.

  if (frq)
    compute(true, frq)
  return _return_loss

PUB calibration(frq)

' Return the open-circuit calibration value for frequency frq.

  return _calibration[frq / 100_000 - 1]
      
PUB compute(resp, frq)

' Compute gamma, return loss, and vswr for frequency frq using
' last sample unless resp == true, in which case a new sample
' is first acquired.

  if (resp == true)
    resp := response(frq)
  _gamma := f.fmin(f.fdiv(f.ffloat(resp), calibration(frq)), 1.0)
  _return_loss := f.fmul(20.0, f.log10(_gamma))
  _vswr := f.fdiv(f.fadd(1.0, _gamma), f.fsub(1.0, _gamma))

PUB response(frq)

' Obtain the raw response for frequency frq.

  outa[nLED]~ 
  set_freq(frq)
  result := adc
  set_freq(frq * 3)
  result := (result - adc / 6) #> 0
  set_freq(0)
  outa[nLED]~~

PRI adc | a, b

' Obtain the delta-sigma ADC value for the input over a 20 ms sample time.
' Subtract the previously-obtained base (zero) value.

  waitcnt(cnt + clkfreq / 100)
  a := phsb
  waitcnt(cnt + clkfreq / 50)
  b := phsb
  return b - a - base   

PRI set_freq(frq) | c

' Set output frequency to frq, unless frq == 0, in which case
' turn off the counter.

  if (frq == 0)
    ctra := 0
  elseif (frq < 500_000)
    ctra := %00100 << 26 | FOUT
  else
    c := %00010_011
    repeat while frq > 8_000_000
      frq >>= 1
      c++
    repeat while frq < 4_000_000
      frq << = 1
      c--
    ctra := c << 23 | FOUT
  frqa := frq_val(frq)

PRI frq_val(frq) : r

' Compute the required value of frqx to obtain the frequency frq.
 
  repeat 32
     frq <<= 1
     r <<= 1
     if frq => clkfreq
        frq -= clkfreq
        r++

PRI squealer

' Send a tone to the audio output, determined by the value of tone. 

  dira := 1 << AUDIO | 1 << LPF
  ctra := %00100 << 26 | AUDIO
  repeat
    frqa := frq_val(tone)      

'-------[ EEPROM Read/Write Routines.]-----------------------------------------

PUB ee_rdlong(addr) | i

' Read a word from EEPROM. Addr needs to be on 4-byte boundary.

  _i2c_waddr(addr)
  _i2c_start
  _i2c_wr(%1010_000_1)
  repeat i from 0 to 3
    result := (result | _i2c_rd(i < 3)) -> 8
  _i2c_stop

PUB ee_wrlong(addr, data)

' Write a word to EEPROM. Addr needs to be on 4-byte boundary.

  result~~
  _i2c_waddr(addr)
  repeat 4
    result &= _i2c_wr(data)
    data >>= 8
  _i2c_stop

PUB ee_rdbyte(addr)

' Read a byte from EEPROM at addr.

  _i2c_waddr(addr)
  _i2c_start
  _i2c_wr(%1010_000_1)
  result := _i2c_rd(NAK)
  _i2c_stop

PUB ee_wrbyte(addr, data)

' Write a byte to EEPROM at addr.

  _i2c_waddr(addr)
  result := _i2c_wr(data)
  _i2c_stop

'-------[ I2C Routines. ]------------------------------------------------------

PRI _i2c_waddr(addr)

  repeat
    _i2c_start
  until _i2c_wr(%1010_000_0) == ACK
  _i2c_wr(addr >> 8)
  _i2c_wr(addr & $ff)

PRI _i2c_rd(acknak)

  repeat 8
    outa[SCL]~~
    result := result << 1 | ina[SDA]
    outa[SCL]~
  dira[SDA] := acknak <> 0
  outa[SCL]~~
  outa[SCL]~
  dira[SDA]~

PRI _i2c_wr(data)

  repeat 8
    dira[SDA] := (data <<= 1) & $100 == 0
    outa[SCL]~~
    outa[SCL]~
  dira[SDA]~
  outa[SCL]~~
  result := ina[SDA] == 0
  outa[SCL]~

PRI _i2c_start

  dira[SDA]~
  outa[SCL]~~
  dira[SDA]~~
  outa[SCL]~

PRI _i2c_stop

  outa[SCL]~
  dira[SDA]~~
  outa[SCL]~~
  dira[SDA]~

